#!/usr/bin/python
# Lifesaver by Chris Jones <cmsj@tenshu.net>
# GPL v2 only
"""friendfeed.py - FriendFeed class"""

import re
from twitterjson import TwitterJson
from stripper import Stripper

class FriendFeed(TwitterJson):
    """Class that collects data from friendfeed"""
    name = "FriendFeed"
    stripper = None

    urlstub = 'http://friendfeed-api.com/v2/search?q=%s&since_id=%d'
    resultname = 'entries'

    def __init__(self):
        """Class initialiser"""
        TwitterJson.__init__(self)
        self.stripper = Stripper()

    def transform_post(self, post):
        """Take the results we have and transform them into what we want"""
        tmppost = {}
        tmppost['id'] = post['id']
        tmppost['source'] = self.name
        tmppost['from_user'] = post['from']['id']
        tmppost['created_at'] = post['date']
        tmppost['text'] = self.stripper.strip(post['body'])
        tmppost['text'] = re.sub('\n', ' ', tmppost['text'])
        tmppost['text'] = re.sub('[ ]{2,}', ' ', tmppost['text'])

        return(tmppost)

