#!/usr/bin/python
# Lifesaver by Chris Jones <cmsj@tenshu.net>
# GPL v2 only
"""screensaver.py - Class that can be embedded as a screensaver"""

import os
import gtk

class Screensaver(gtk.Window):
    """Class that attaches to a screensaver manager, if running"""
    # pylint: disable-msg=R0904

    __gsignals__ = { 'realize': 'override' }

    def __init__(self):
        """Class initialiser"""
        gtk.Window.__init__(self)

    def fallback(self):
        """Common operations for when we just want to be a gtk.Window"""
        gtk.Window.do_realize(self)
        self.set_size_request(1024, 600)

    def do_realize(self):
        """Search for an XScreensaver to belong to, otherwise strike out
        alone"""
        env = os.getenv('XSCREENSAVER_WINDOW')
        if not env:
            self.fallback()
            return
        
        if env[0:2] == '0x':
            env = env[2:]

        xid = int(env, 16)

        window = gtk.gdk.window_foreign_new(xid)
        if not window:
            self.fallback()
            return

        self.set_decorated(False)

        self.set_property('skip-taskbar-hint', True)
        self.set_property('skip-pager-hint', True)
        self.set_keep_above(True)
        self.fullscreen()

        mask = gtk.gdk.EXPOSURE_MASK | gtk.gdk.STRUCTURE_MASK
        window.set_events(window.get_events() | mask)
        self.set_events(self.get_events() | mask)

        self.window = window
        window.set_user_data(self)
        self.set_flags(gtk.REALIZED)

        geom = window.get_geometry()
        self.size_allocate(gtk.gdk.Rectangle(geom[0], geom[1], 
                                             geom[2], geom[3]))
        self.resize(geom[2], geom[3])

