#!/usr/bin/python
# Lifesaver by Chris Jones <cmsj@tenshu.net>
# GPL v2 only
"""twitterjson.py - Twitter class that searches json"""

import re
import urllib2
import json
from lifesaver import Lifesaver
from util import dbg, htmldecode

class TwitterJson(object):
    """Class that collects data from twitter"""
    name = "Twitter"
    lifesaver = None

    urlstub = 'http://search.twitter.com/search.json?q=%s&rpp=25&since_id=%d'
    resultname = 'results'
    most_recent_id = None
    posts = None
    recent = None

    def __init__(self):
        """Class initialiser"""
        self.lifesaver = Lifesaver()
        self.posts = {}
        self.most_recent_id = 1
        self.recent = []

    def transform_post(self, post):
        """Take the results we have and transform them into what we want"""
        tmppost = {}
        tmppost['id'] = post['id']
        tmppost['source'] = self.name
        tmppost['from_user'] = post['from_user']
        tmppost['created_at'] = post['created_at']
        tmppost['text'] = htmldecode(post['text'])
        tmppost['text'] = re.sub('\n', ' ', tmppost['text'])
        tmppost['text'] = re.sub('[ ]{2,}', ' ', tmppost['text'])
        
        return(tmppost)

    def fetch_data(self):
        """Attempt to fetch more data from the source"""
        dbg(self.name, 'fetch_data: started with %d items' % len(self.posts))
        for keyword in self.lifesaver.config['keywords']:
            dbg(self.name, 'fetch_data: fetching data for keyword: %s (>%d)' %
                    (keyword, self.most_recent_id))
            try:
                results = urllib2.urlopen(self.urlstub % 
                        (keyword, self.most_recent_id))
            except (urllib2.HTTPError, urllib2.URLError):
                dbg(self.name, 'fetch_data: urllib error, probably no/slow \
                        internet')
                continue

            try:
                data = json.load(results)
            except ValueError:
                data = {self.resultname:[]}

            dbg(self.name, 'fetch_data: processing %d results' %
                len(data[self.resultname]))
            for post in data[self.resultname]:
                tmppost = self.transform_post(post)
                tmppost['keyword'] = keyword

                if self.posts.has_key(tmppost['id']):
                    continue
                else:
                    self.posts[tmppost['id']] = tmppost

            if data.has_key('max_id') and data['max_id'] > self.most_recent_id:
                self.most_recent_id = data['max_id']

        dbg(self.name, 'fetch_data: finished with %d items' % len(self.posts))
        if len(self.posts) > 250:
            dbg(self.name, 'fetch_data: more than 250 posts, truncating')
            del(self.posts[250:len(self.posts)])

    def has_items(self):
        """Query whether or not we have any data to give"""
        return(len(self.posts) > 0)

    def get_item(self):
        """Return a single interesting item"""
        if len(self.posts) == 0:
            return({'from_user':None, 
                    'created_at':None, 
                    'text':'Still loading data',
                    'keyword':'', 'source':''})
        else:
            keys = self.posts.keys()
            keys.sort()
            keys.reverse()

            for key in keys:
                if not key in self.recent:
                    self.recent.append(key)
                    return(self.posts[key])

            # We've failed to find anything new, nuke self.recent so we cycle
            dbg(self.name, 'get_item: resetting self.recent')
            self.recent = []
            return(self.posts[keys[0]])

